NmTimerUpdater subclass: #NmWorldMapLocationBroadcaster
	instanceVariableNames: 'broadcastClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NetMorph-Server-MapServer'!

!NmWorldMapLocationBroadcaster methodsFor: 'accessing' stamp: 'mu 5/29/2005 03:00'!
broadcastClient
	broadcastClient ifNil: [broadcastClient _ RbtClient new].
	^broadcastClient! !


!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 5/29/2005 02:48'!
broadcastingIntervalSeconds
	^(self intervalSeconds // self broadcastingPorts size) max: 1! !

!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 6/3/2005 03:23'!
broadcastingPorts
	^NmServerSettings broadcastingWarpPorts! !

!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 5/29/2005 03:52'!
locationMessage
	^NmBaseUtil localAddress, ':', NmServerSettings mapServerPort asString! !


!NmWorldMapLocationBroadcaster methodsFor: 'actions' stamp: 'mu 6/21/2005 18:52'!
update
	NmServerManager isMapServerRunning ifFalse: [^self].
	self broadcastingPorts
		do: [:port | 
			self broadcastClient bindForBroadcast: port.
			self broadcastClient sendData: self locationMessage.
			(Delay forSeconds: self broadcastingIntervalSeconds) wait]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NmWorldMapLocationBroadcaster class
	instanceVariableNames: ''!

!NmWorldMapLocationBroadcaster class methodsFor: 'constants' stamp: 'mu 5/30/2005 18:52'!
defaultIntervalSeconds
	^ 12! !


NmSoapServer subclass: #NmWorldMapServer
	instanceVariableNames: 'worldMaps clearProcess storeProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NetMorph-Server-MapServer'!

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 6/15/2005 23:54'!
backupFilename
	^NmBaseUtil shortImagePrefix, '.AllWorldMaps.bak'! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 2/2/2005 01:03'!
clearingIntervalSeconds
	^self class defaultClearingIntervalSeconds! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 6/15/2005 23:43'!
storingIntervalSeconds
	^self class defaultStoringIntervalSeconds! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 1/21/2005 01:16'!
worldMaps
	worldMaps ifNil: [worldMaps _ Dictionary new].
	^worldMaps! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 1/21/2005 02:20'!
worldMapsAt: mapName 
	^ self worldMaps
		at: mapName
		ifAbsentPut: [NmWorldMap new]! !


!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 5/4/2005 18:48'!
clearMapsNotUsed
	self clearMapsNotUsedFor: self clearingIntervalSeconds.
	! !

!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 6/15/2005 00:06'!
clearMapsNotUsedFor: intervalSeconds 
	| seconds emptyMaps emptyMapKeys |
	seconds _ intervalSeconds.
	self worldMaps
		do: [:each | each removeNotUpdatedFor: seconds].
	emptyMaps _ self worldMaps
				select: [:each | each isEmpty].
	emptyMaps
		keysAndValuesDo: [:name :map | map forgetNamed: name useServer: false].
	emptyMapKeys _ emptyMaps keys.
	emptyMapKeys
		do: [:each | self worldMaps
				removeKey: each
				ifAbsent: []]! !

!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 5/10/2005 17:39'!
clingTogether
	self worldMaps
		do: [:each | 
			each clingTogetherAloneEntries].! !


!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 1/21/2005 02:03'!
getMapStringNamed: mapName 
	| map |
	map _ self worldMaps
				at: mapName
				ifAbsent: [].
	^ map isNil
		ifFalse: [map csvString]! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/7/2005 10:22'!
getVersionNamed: mapName 
	| map |
	map _ self worldMaps
				at: mapName
				ifAbsent: [].
	^ map isNil
		ifFalse: [map version]! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/4/2005 02:29'!
listAllMapNames
	self clearMapsNotUsed.
	^self worldMaps keys asSortedCollection asArray
	
	! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/7/2005 10:51'!
putMapString: mapCSVString named: mapName 
	| map |
	map _ self worldMapsAt: mapName.
	map _ map fromCsvString: mapCSVString.
	map mapName: mapName.
	map incrementVersion.
	map storeNamed: mapName useServer: false.
	^ nil! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 2/2/2005 00:30'!
removeEntryWith: uuidStr named: mapName
	| map |
	map _ self worldMapsAt: mapName.
	map
		removeKey: uuidStr
		ifAbsent: [^ false].
	map storeNamed: mapName useServer: false.
	^ true! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 7/6/2005 02:15'!
updateEntryWith: mapEntryString named: mapName 
	| worldMap origMap newMap |
	self clearMapsNotUsed.
	worldMap _ self worldMapsAt: mapName.
	origMap _ worldMap map copy.
	worldMap
		registerDodgeIfConflict: ((NmWorldMapEntry fromCsvString: mapEntryString) unchanged).
	newMap _ worldMap map.
	origMap = newMap
		ifFalse: [worldMap incrementVersion.
			worldMap storeNamed: mapName useServer: false].
	^ worldMap csvString! !


!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 6/15/2005 23:12'!
initClearProcess
	| cleaner |
	clearProcess notNil
		ifTrue: [self release].
	cleaner _ [[self clearMapsNotUsed.
			self clingTogether.
			(Delay forSeconds: self clearingIntervalSeconds) wait] repeat].
	clearProcess _ cleaner forkAt: Processor userSchedulingPriority - 2! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 6/15/2005 23:58'!
initialize
	"self default initialize"
	super initialize.
	self release.
	worldMaps _ nil.
	self initClearProcess.
	self initStoreProcess! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 6/15/2005 23:43'!
initStoreProcess
	| saver |
	storeProcess notNil
		ifTrue: [self release].
	saver _ [[self store.
			(Delay forSeconds: self storingIntervalSeconds) wait] repeat].
	storeProcess _ saver forkAt: Processor userSchedulingPriority - 5! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 6/15/2005 23:41'!
release
	clearProcess ifNotNilDo: [:p | p terminate].
	clearProcess _ nil.
	storeProcess ifNotNilDo: [:p | p terminate].
	storeProcess _ nil
	! !


!NmWorldMapServer methodsFor: 'storing/loading' stamp: 'mu 6/15/2005 23:56'!
load
	| loaded |
	loaded _ NmBaseUtil loadFrom: FileDirectory default named: self backupFilename.
	loaded notNil
		ifTrue: [worldMaps _ loaded.
			NmTranscript crShow: '## All MapServer data were loaded: ' , self backupFilename]! !

!NmWorldMapServer methodsFor: 'storing/loading' stamp: 'mu 6/16/2005 18:58'!
store
	self worldMaps isEmpty ifTrue: [^self].
	NmBaseUtil store: self worldMaps to: FileDirectory default named: self backupFilename! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NmWorldMapServer class
	instanceVariableNames: ''!

!NmWorldMapServer class methodsFor: 'constants' stamp: 'mu 2/2/2005 01:01'!
defaultClearingIntervalSeconds
	^20! !

!NmWorldMapServer class methodsFor: 'constants' stamp: 'mu 6/16/2005 00:57'!
defaultStoringIntervalSeconds
	"1 hour"
	^60 * 60
	! !


!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 4/10/2005 02:08'!
exploreMaps
	"NmWorldMapServer exploreMaps"
	self default worldMaps explore! !

!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 6/15/2005 23:39'!
load
	"NmWorldMapServer load"
	^self default load! !

!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 2/2/2005 00:51'!
release
	"self release"
	self default release! !

!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 6/15/2005 23:56'!
store
	"NmWorldMapServer store"
	^self default store! !


!NmWorldMapServer class methodsFor: 'class initialization' stamp: 'mu 3/12/2003 00:19'!
initialize
	self basicInitialize! !

!NmWorldMapServer class methodsFor: 'class initialization' stamp: 'mu 2/2/2005 00:52'!
obsolete
	self release.
	super obsolete! !


!NmWorldMapServer class methodsFor: 'registering' stamp: 'mu 5/7/2005 10:23'!
registerServices
	"NmWorldMapServer registerServices"
	self registerServiceNamed: #'putMapString:named:' paramNames: #(#mapCSVString #mapName ).
	self registerServiceNamed: #'getMapStringNamed:' paramNames: #(#mapName ).
	self registerServiceNamed: #'getVersionNamed:' paramNames: #(#mapName ).
	self registerServiceNamed: #listAllMapNames paramNames: #().
	self registerServiceNamed: #'updateEntryWith:named:' paramNames: #(#mapEntryString #mapName ).
	self registerServiceNamed: #'removeEntryWith:named:' paramNames: #(#uuidStr #mapName )! !

NmWorldMapServer initialize!